<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;

class Banco extends \Zeedhi\Framework\Controller\Simple {

	protected $salesMessage;
    protected $bancoService;
	protected $util;

	public function __construct(Services\Banco $bancoService, Utilities\SalesMessage $salesMessage, Utilities\Util $util){
		$this->bancoService = $bancoService;
		$this->salesMessage = $salesMessage;
        $this->util = $util;
	}

	public function buscaBancos(Request\Filter $request, Response $response) {
        try {
            $bancos = $this->bancoService->buscaBancos();

            if (empty($bancos)) {
                $response->addMessage(new Message($this->salesMessage->getMessage(30)));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BancosRepo', array()));
            } else {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BancosRepo', $bancos));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function buscaBancoEspecifico(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $CDBANCO = $params[0]['value'];

            $banco = $this->bancoService->buscaBancoEspecifico($CDBANCO);
            $mascara = $this->bancoService->buscaMascara($CDBANCO);
            if (empty($banco)){
                //retorna apenas o codigo do banco
                 $banco = $this->bancoService->buscaBanco($CDBANCO);
                 $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BancoEspecificoRepo', $banco));
            } else {
                    if (empty($mascara)) {
                        // máscara não encontrada
                        $response->addMessage(new Message($this->salesMessage->getMessage(38)));
                    } else {
                        $banco = $this->mascara($banco, $mascara[0]['CDPICTAGEN'], 'NMAGENCIA');
                        $banco = $this->mascara($banco, $mascara[0]['CDPICTCONT'], 'CDCONTCORR');
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BancoEspecificoRepo', $banco));
                    }
                }
            }
         catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function atualizaBanco(Request\Filter $request, Response $response) {
        try {
            $params         = $request->getFilterCriteria()->getConditions();

            $CDBANCO        = $params[0]['value'];
            $DSINSCEMPVND   = $params[1]['value'];
            $QTDIASVENVND   = $params[2]['value'];
            $CDAGENCIAVND   = $params[3]['value'];
            $CDCONTCORVND   = $params[4]['value'];
            $CDCONTRATOVND  = $params[5]['value'];
            $CDBANCOCONVVND    = $params[6]['value'];
            if($DSINSCEMPVND == ""){
            	$DSINSCEMPVND = null;
            }
            if($QTDIASVENVND == ""){
            	$QTDIASVENVND = null;
            }
            if($CDAGENCIAVND == ""){
            	$CDAGENCIAVND = null;
            }
            if($CDCONTCORVND == ""){
            	$CDCONTCORVND = null;
            }
            if($CDCONTRATOVND == ""){
            	$CDCONTRATOVND = null;
            }
            if($CDBANCOCONVVND == ""){
            	$CDBANCOCONVVND = null;
            }

            $banco = $this->bancoService->atualizaBanco($CDBANCO, $DSINSCEMPVND, $QTDIASVENVND, $CDAGENCIAVND, $CDCONTCORVND, $CDCONTRATOVND, $CDBANCOCONVVND);

            $response->addMessage(new Message($this->salesMessage->getMessage("atualizado")));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('nothing', array(array('nothing' => 'nothing'))));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function buscaAgencias(Request\Filter $request, Response $response) {
        try {
            $params  = $request->getFilterCriteria()->getConditions();
            $CDBANCO = $params[0]['value'];

            $agencias = $this->bancoService->buscaAgencias($CDBANCO);
            $mascara = $this->bancoService->buscaMascara($CDBANCO);

            if (empty($agencias)){
                // Não há agências cadastradas neste banco.
                $response->addMessage(new Message($this->salesMessage->getMessage(32)));
            } else {
                 if (empty($mascara)) {
                    // máscara não encontrada
                    $response->addMessage(new Message($this->salesMessage->getMessage(38)));
                } else {
                    $agencias = $this->mascara($agencias, $mascara[0]['CDPICTAGEN'], 'NMAGENCIA');
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaAgenciasRepo', $agencias));
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function buscaContas(Request\Filter $request, Response $response) {
        try {
            $params    = $request->getFilterCriteria()->getConditions();
            $CDBANCO   = $params[0]['value'];
            $CDAGENCIA = $params[1]['value'];

            $contas = $this->bancoService->buscaContas($CDBANCO, $CDAGENCIA);
            $mascara = $this->bancoService->buscaMascara($CDBANCO);

            if (empty($contas)) {
                //Não há contas cadastradas nesta agência.
                $response->addMessage(new Message($this->salesMessage->getMessage(33)));
            } else {
                if (empty($mascara)) {
                    // máscara não encontrada
                    $response->addMessage(new Message($this->salesMessage->getMessage(38)));
                } else {
                    $contas = $this->mascara($contas, $mascara[0]['CDPICTCONT'], 'CDCONTCORR');
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaContasRepo', $contas));
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function mascara($array, $mascara, $campo){
        $ndigitos = strpos($mascara, '-');
        if (($ndigitos) == false){
            return $array;
        } else {
            $idx = 0;
            while ($idx < sizeof($array)) {
                $contaAtual = $array[$idx][$campo];
                $array[$idx][$campo] = substr($contaAtual, 0, $ndigitos) . '-' . substr($contaAtual, ($ndigitos));
                $idx++;
            }
        }
        return $array;
    }

    public function getBlockedCards(Request\Filter $request, Response $response){
        try {
            $blockedCards = $this->bancoService->getBlockedCards();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BlockedCardsRepository', $blockedCards));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function addBlockedCard(Request\Filter $request, Response $response){
        try {
            $params    = $request->getFilterCriteria()->getConditions();
            $NRPRIMEIROS = $params[0]['value'];
            $NRULTIMOS   = $params[1]['value'];
            $DESCCARTAO  = $params[2]['value'];

            if (!is_numeric($NRPRIMEIROS) || !is_numeric($NRULTIMOS)){
                throw new \Exception("Favor preencher os dados obrigatórios com valores númericos.");
            }
            else {
                if ($this->cardExists(null, $NRPRIMEIROS, $NRULTIMOS)){
                    throw new \Exception("O cartão informado já está cadastrado na lista.");
                }
                else {
                    /* Adds a new card to the table. */
                    $NRSEQCARTAOBLOQ = $this->util->newCode('CARTAOBLOQ', 10);
                    $this->bancoService->addBlockedCard($NRSEQCARTAOBLOQ, $NRPRIMEIROS, $NRULTIMOS, $DESCCARTAO);
                    /* Gets an updated list of cards to send back. */
                    $blockedCards = $this->bancoService->getBlockedCards();
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BlockedCardsRepository', $blockedCards));
                }
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function updateBlockedCard(Request\Filter $request, Response $response){
        try {
            $params    = $request->getFilterCriteria()->getConditions();
            $NRSEQCARTAOBLOQ = $params[0]['value'];
            $NRPRIMEIROS = $params[1]['value'];
            $NRULTIMOS   = $params[2]['value'];
            $DESCCARTAO  = $params[3]['value'];

            if (!is_numeric($NRPRIMEIROS) || !is_numeric($NRULTIMOS)){
                throw new \Exception("Favor preencher os dados obrigatórios com valores númericos.");
            }
            else {
                if ($this->cardExists($NRSEQCARTAOBLOQ, $NRPRIMEIROS, $NRULTIMOS)){
                    throw new \Exception("O cartão informado já está cadastrado na lista.");
                }
                /* Updates the blocked card. */
                $this->bancoService->updateBlockedCard($NRSEQCARTAOBLOQ, $NRPRIMEIROS, $NRULTIMOS, $DESCCARTAO);
                /* Gets an updated list of cards to send back. */
                $blockedCards = $this->bancoService->getBlockedCards();
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BlockedCardsRepository', $blockedCards));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function deleteBlockedCard(Request\Filter $request, Response $response){
        try {
            $params    = $request->getFilterCriteria()->getConditions();
            $NRSEQCARTAOBLOQ = $params[0]['value'];

            /* Deletes the blocked card. */
            $this->bancoService->deleteBlockedCard($NRSEQCARTAOBLOQ);
            /* Gets an updated list of cards to send back. */
            $blockedCards = $this->bancoService->getBlockedCards();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BlockedCardsRepository', $blockedCards));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    /* This function checks to see if a card exists in the blocked cards table. */
    public function cardExists($NRSEQCARTAOBLOQ, $NRPRIMEIROS, $NRULTIMOS){
        $blockedCard = $this->bancoService->getBlockedCard($NRPRIMEIROS, $NRULTIMOS);
        if (!empty($blockedCard)){
            return !($blockedCard[0]['NRSEQCARTAOBLOQ'] == $NRSEQCARTAOBLOQ);
        }
        else return false;
    }

}