<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelatoriosTotAcProdComb {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function find($conditions) {
        $params = array(
         	":CDFILIAL" => $conditions['CDFILIAL'],
         	":TCDFILIAL" => $conditions['TCDFILIAL'],
         	":CDLOJA" => $conditions['CDLOJA'],
         	":TCDLOJA" => $conditions['TCDLOJA'],
         	":DTVENDAINI" => $conditions['DTVENDAINI'],
         	":DTVENDAFIN" => $conditions['DTVENDAFIN'],
         	":MODALIDADE" => $conditions['MODALIDADE'],
         	":TMODALIDADE" => $conditions['TMODALIDADE'],
         
         	":CDARVPRODF" => $conditions['CDARVPRODF'],
         	":CDARVPRODI" => $conditions['CDARVPRODI'],
         	":CDPRODLIST" => $conditions['CDPRODLIST'],
         	":PRODNIV" => $conditions['PRODNIV'],
         	":TCDPRODUTO" => $conditions['TCDPRODUTO'],
         	
         	":NRORG" => $conditions['NRORG']
 		);
 		return $this->entityManager->getConnection()->fetchAll(self::GET_TOT_PROD_COMB, $params);
    }
    
    public function buildParamsFaturamento($conditions){    
        $conditions['CDFILIAL'] = implode('_', $conditions['CDFILIAL']);
        $conditions['TCDFILIAL'] = 'F';
        if( array_key_exists('CDLOJA', $conditions) ) {
            $conditions['CDLOJA'] = implode($conditions['CDLOJA'], '_');
            $conditions['TCDLOJA'] = 'F';
        }  
        else {
            $conditions['CDLOJA'] = null;
            $conditions['TCDLOJA'] = 'T';    
        }
        if( array_key_exists('MODALIDADE', $conditions) ) {
            $conditions['MODALIDADE'] = implode($conditions['MODALIDADE'], '_');
            $conditions['TMODALIDADE'] = 'F';
        }  
        else {
            $conditions['MODALIDADE'] = null;
            $conditions['TMODALIDADE'] = 'T';    
        }
        if(array_key_exists('DTENTRVENDA', $conditions)) {
            $conditions['DTVENDAINI'] = $conditions['DTENTRVENDA'][0];
            $conditions['DTVENDAFIN'] = $conditions['DTENTRVENDA'][1];
            unset($conditions['DTENTRVENDA']);
        }
        if (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'I'){
           if (empty($conditions['CDARVPRODF'])){
                $conditions['CDARVPRODF'] = $conditions['CDARVPRODI'];
           }
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDPRODLIST'] = null;
           $conditions['PRODNIV'] = null;
        }elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'D') {
           $conditions['PRODNIV'] = str_replace('%', '', str_replace('.', '', $conditions['PRODNIV'])).'%';
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDPRODLIST'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
        }
        elseif (isset($conditions['INTERVALISTA']) && $conditions['INTERVALISTA'] == 'L'){
           $conditions['CDPRODLIST'] = implode('_', $conditions['CDPRODUTO']);
           $conditions['TCDPRODUTO'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
           $conditions['PRODNIV'] = null;
        }
        else{
           $conditions['TCDPRODUTO'] = 'T';
           $conditions['CDPRODLIST'] = null;
           $conditions['CDARVPRODF'] = null;
           $conditions['CDARVPRODI'] = null;
           $conditions['PRODNIV'] = null;
        }
        return $conditions;
    }

    const GET_TOT_PROD_COMB = "
        SELECT PR.CDPRODUTO, PR.CDARVPROD, FILOCAVE.CDFILIAL, FILOCAVE.CDFILIAL || ' - ' || FILOCAVE.NMFILIAL AS LABELFILIAL, PR.NMPRODUTO, PR.SGUNIDADE, FILOCAVE.NRORG,
               G.PRODPAI, PP.NMPRODUTO AS NMPRODPAI, G.CDGRUPROMOC, G.NMGRUPROMOC, FILOCAVE.LABELMODALIDADE, G.PRODPAI||' - '||PP.NMPRODUTO AS LABELPRODPAI,
               G.CDGRUPROMOC||' - '||G.NMGRUPROMOC AS LABELGRUPO,
               SUM(TRUNC( IT.QTPRODVEND * (IT.VRUNITVEND + NVL( IT.VRUNITVENDCL, 0 ) ), 2 ) - NVL( IT.VRDESITVEND, 0 ) + NVL( IT.VRACRITVEND, 0 ) ) AS TOTALVEND,
               SUM(IT.QTPRODVEND) AS QTPRODVEND, SUM(NVL( IT.VRDESITVEND, 0 ) ) AS VRDESITVEND, SUM(NVL( IT.VRACRITVEND, 0 ) ) AS ACRESCIMO
          FROM ITEMVENDA IT
          JOIN PRODUTO PR 
            ON IT.CDPRODUTO = PR.CDPRODUTO
           AND IT.CDPRODUTO = PR.CDPRODUTO
          JOIN ( SELECT V.IDSITUVENDA, V.NRSEQVENDA, FILOCA.CDCAIXA, FILOCA.NMCAIXA, FILOCA.CDLOJA, FILOCA.NMLOJA, FILOCA.CDFILIAL, FILOCA.NMFILIAL,
                        FILOCA.NRORG, FILOCA.CDPRODTAXASERV, CASE SUBSTR(V.IDORIGEMVENDA, 0, 3)
                                                 WHEN 'DLV' THEN 'DELIVERY'
                                                 WHEN 'MES' THEN 'MESA'
                                                 WHEN 'BAL' THEN (CASE SUBSTR(V.IDORIGEMVENDA, 5, 7) WHEN 'AUT' THEN 'TERMINAL DE AUTO-ATENDIMENTO' ELSE 'BALCAO' END) 
                                                 WHEN 'TAA' THEN 'TERMINAL DE AUTO-ATENDIMENTO'
                                                 WHEN 'CMD' THEN 'COMANDA'
                                                 WHEN 'AJU' THEN 'OUTROS'
                                                 ELSE 'OUTROS'
                                               END AS LABELMODALIDADE
                   FROM VENDA V
                   JOIN ( SELECT C.CDCAIXA, C.NMCAIXA, FILO.CDLOJA, FILO.NMLOJA, FILO.CDFILIAL, FILO.NMFILIAL, FILO.NRORG, FILO.CDPRODTAXASERV
                            FROM CAIXA C 
                            JOIN ( SELECT L.CDLOJA, L.NMLOJA, FI.CDFILIAL, FI.NMFILIAL, FI.NRORG, L.CDPRODTAXASERV 
                                     FROM LOJA L 
                                     JOIN ( SELECT F.CDFILIAL, F.NMFILIAL, F.NRORG
                                              FROM FILIAL F 
                                             WHERE ( ( INSTR( :CDFILIAL, F.CDFILIAL ) <> 0 ) OR ( :TCDFILIAL = 'T' ) ) 
                                               AND F.NRORG =:NRORG
                                          ) FI 
                                       ON L.CDFILIAL = FI.CDFILIAL
                                      AND L.NRORG = FI.NRORG
                                      AND ( ( INSTR( :CDLOJA, L.CDFILIAL||L.CDLOJA ) <> 0 ) OR ( :TCDLOJA = 'T' ) )
                                 ) FILO 
                              ON C.CDFILIAL = FILO.CDFILIAL
                             AND C.CDLOJA = FILO.CDLOJA
                             AND C.NRORG = FILO.NRORG
                         ) FILOCA 
                      ON V.CDCAIXA = FILOCA.CDCAIXA
                     AND V.CDLOJA = FILOCA.CDLOJA
                     AND V.CDFILIAL = FILOCA.CDFILIAL
                     AND V.NRORG = FILOCA.NRORG
                   WHERE V.IDSITUVENDA = 'O'
                     AND V.DTENTRVENDA BETWEEN :DTVENDAINI AND :DTVENDAFIN
                     AND (INSTR(:MODALIDADE, SUBSTR(V.IDORIGEMVENDA, 0, 3) ) > 0 OR (:TMODALIDADE = 'T'))
               ) FILOCAVE 
            ON FILOCAVE.CDFILIAL = IT.CDFILIAL
           AND FILOCAVE.CDLOJA = IT.CDLOJA
           AND FILOCAVE.CDCAIXA = IT.CDCAIXA
           AND FILOCAVE.NRSEQVENDA = IT.NRSEQVENDA
           AND FILOCAVE.NRORG = IT.NRORG
          JOIN ( SELECT PGM.CDPRODUTO AS PRODPAI, PGM.NRORG, PGM.CDGRUPROMOC, GM.NMGRUPROMOC
                   FROM PRODGRUPROMOC PGM  
                   JOIN GRUPROMOC GM
                     ON PGM.CDGRUPROMOC = GM.CDGRUPROMOC 
                    AND PGM.NRORG = GM.NRORG ) G
            ON IT.CDPRODPROMOCAO = G.PRODPAI
           AND IT.NRORG = G.NRORG
          JOIN PRODUTO PP
            ON G.PRODPAI = PP.CDPRODUTO
           AND IT.CDPRODPROMOCAO = PP.CDPRODUTO
           AND G.NRORG = PP.NRORG
         WHERE IT.CDPRODPROMOCAO IS NOT NULL
           AND (((((INSTR(:CDPRODLIST, PR.CDPRODUTO) > 0 ) OR (PR.CDARVPROD BETWEEN :CDARVPRODI AND :CDARVPRODF ))) OR (PR.CDARVPROD LIKE :PRODNIV)) OR (:TCDPRODUTO = 'T'))
         GROUP BY PR.CDPRODUTO, FILOCAVE.CDFILIAL, FILOCAVE.CDFILIAL, FILOCAVE.NMFILIAL, PR.CDARVPROD, PR.SGUNIDADE, PR.NMPRODUTO, IT.VRUNITVENDCL, FILOCAVE.NRORG, G.PRODPAI, PP.NMPRODUTO, G.CDGRUPROMOC, G.NMGRUPROMOC, FILOCAVE.LABELMODALIDADE
         ORDER BY FILOCAVE.CDFILIAL, G.PRODPAI, PR.CDARVPROD
    ";
}